@@ -1,10 +1,12 @@
 ;;; Guile-Git --- GNU Guile bindings of libgit2
 ;;; Copyright © 2016 Amirouche Boubekki <amirouche@hypermove.net>
 ;;; Copyright © 2016, 2017 Erik Edrosa <erik.edrosa@gmail.com>
-;;; Copyright © 2017, 2019, 2020, 2021 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2017, 2019-2021, 2024 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2017 Mathieu Othacehe <m.othacehe@gmail.com>
 ;;; Copyright © 2018 Jelle Licht <jlicht@fsfe.org>
 ;;; Copyright © 2019 Marius Bakke <marius@devup.no>
+;;; Copyright © 2022 Maxime Devos <maximedevos@telenet.be>
+;;; Copyright © 2023 Nicolas Graves <ngraves@ngraves.fr>
 ;;;
 ;;; This file is part of Guile-Git.
 ;;;
@@ -66,8 +68,10 @@
             make-status-options-bytestructure status-options->pointer set-status-options-show! set-status-options-flags!
 
             make-remote-callbacks remote-callbacks->pointer set-remote-callbacks-version! set-remote-callbacks-transfer-progress!
+            set-remote-callbacks-certificate-check!
             make-fetch-options-bytestructure fetch-options-bytestructure fetch-options->pointer fetch-options-remote-callbacks
             fetch-options-download-tags set-fetch-options-download-tags!
+            fetch-options-depth set-fetch-options-depth!
             set-remote-callbacks-credentials!
             fetch-options-proxy-options
 
@@ -86,6 +90,7 @@
             set-proxy-options-url! set-proxy-options-type!
 
 
+            make-checkout-options-bytestructure checkout-options-bytestructure checkout-options->pointer
             make-clone-options-bytestructure clone-options-bytestructure clone-options->pointer set-clone-options-fetch-opts!
             submodule-update-options-bytestructure
             make-submodule-update-options-bytestructure
@@ -134,8 +139,9 @@
 
 ;;; git-time
 
-(define %time (bs:struct `((time ,int64) ;; time in seconds since epoch
-                           (offset ,int)))) ;; timezone offset, in minutes
+(define %time (bs:struct `((time ,int64)         ;time in seconds since epoch
+                           (offset ,int)         ;timezone offset, in minutes
+                           (sign ,int8))))       ;sign for "-0000" offsets
 
 (define-record-type <time>
   (%make-time bytestructure)
@@ -191,12 +197,12 @@
   oid?
   (bytevector oid-bytevector))
 
-(define (pointer->oid pointer)
+(define* (pointer->oid pointer #:optional (offset 0))
   ;; C functions typically return 'const git_oid *' and the OID's memory
   ;; belongs to the object it is associated with.  Thus, always copy the OID
   ;; contents to make sure it's not modified or freed behind our back.
   (%make-oid (bytevector-copy
-              (pointer->bytevector pointer GIT-OID-RAWSZ))))
+              (pointer->bytevector pointer GIT-OID-RAWSZ offset))))
 
 (define (oid->pointer oid)
   (bytevector->pointer (oid-bytevector oid)))
@@ -225,7 +231,11 @@
   (bs:struct `((version ,unsigned-int)
                (status-show ,int)
                (flags ,unsigned-int)
-               (pathspec ,%strarray))))
+               (pathspec ,%strarray)
+               (baseline ,(bs:pointer void))
+               ,@(if %have-status-options-rename-threshold?
+                     `((rename-threshold ,uint16))
+                     '()))))
 
 (define %diff-file
   (bs:struct `((oid ,(bs:vector 20 uint8))
@@ -486,7 +496,7 @@
 
 (define %config-entry (bs:struct `((name ,(bs:pointer uint8)) ; char *
                                    (value ,(bs:pointer uint8)) ; char *
-                                   (include-depth ,uint64)
+                                   (include-depth ,unsigned-int)
                                    (level ,int) ; git_config_level_t
                                    (free ,(bs:pointer int))
                                    (payload ,(bs:pointer int)))))
@@ -637,13 +647,9 @@ type to 'specified for this to take effect."
                (push-update-reference ,(bs:pointer uint8))
                (push-negotiation ,(bs:pointer uint8))
                (transport ,(bs:pointer uint8))
+               (remote-ready ,(bs:pointer void))
                (payload ,(bs:pointer uint8))
-
-               ;; libgit2 1.0 added this field, which is missing from 0.28.5,
-               ;; even though in both cases GIT_REMOTE_CALLBACKS_VERSION = 1.
-               ,@(if %have-remote-callbacks-resolve-url?
-                     `((resolve-url ,(bs:pointer uint8)))
-                     '()))))
+               (resolve-url ,(bs:pointer uint8)))))
 
 (define-record-type <remote-callbacks>
   (%make-remote-callbacks bytestructure)
@@ -670,8 +676,17 @@ type to 'specified for this to take effect."
                (update-fetchhead ,int)
                (download-tags ,int)
                (proxy-opts ,%proxy-options)
+               ,@(if %have-fetch-options-depth?
+                     `((depth ,int))     ;one of the GIT-FETCH-DEPTH-* values
+                     '())
+               ,@(if %have-fetch-options-follow-redirects?
+                     `((follow-redirects ,int))
+                     '())
                (custom-headers ,%strarray))))
 
+(define GIT-FETCH-DEPTH-FULL 0)
+(define GIT-FETCH-DEPTH-UNSHALLOW 2147483647)
+
 (define-record-type <fetch-options>
   (%make-fetch-options bytestructure)
   fetch-options?
@@ -718,6 +733,33 @@ tag policy in FETCH-OPTIONS."
                       'download-tags
                       (symbol->remote-autotag-option policy)))
 
+(define (fetch-depth->symbol depth)
+  (cond ((= depth GIT-FETCH-DEPTH-UNSHALLOW) 'unshallow)
+        (else 'full)))
+
+(define (symbol->fetch-depth symbol)
+  (case symbol
+    ((unshallow) GIT-FETCH-DEPTH-UNSHALLOW)
+    ((full)      GIT-FETCH-DEPTH-FULL)
+    (else        GIT-FETCH-DEPTH-FULL)))
+
+(define (fetch-options-depth fetch-options)
+  "Return the \"depth\" parameter of FETCH-OPTIONS as a symbol: 'full or
+'unshallow.  On libgit2 < 1.7, this is always 'full."
+  (if %have-fetch-options-depth?
+      (fetch-depth->symbol
+       (bytestructure-ref (fetch-options-bytestructure fetch-options)
+                          'depth))
+      'full))
+
+(define (set-fetch-options-depth! fetch-options depth)
+  "Change FETCH-OPTIONS depth parameter to DEPTH, one of 'full (obtain the
+full repository history), and 'unshallow (fetch missing data from a shallow
+repository)."
+  (when %have-fetch-options-depth?
+    (bytestructure-set! (fetch-options-bytestructure fetch-options)
+                        'depth (symbol->fetch-depth depth))))
+
 (define fetch-options-remote-callbacks
   (let ((cache (make-weak-key-hash-table 20)))
     (lambda (fetch-options)
@@ -772,13 +814,39 @@ indexer progress record.  PROC can cancel the on-going transfer by returning
                         'transfer-progress
                         (pointer-address ptr))))
 
+(define (set-remote-callbacks-certificate-check! callbacks proc)
+  "Use PROC as the certificate check procedure in CALLBACKS.  PROC will be
+called with two arguments: the host name and a boolean indicating whether
+libgit2 thinks the certificate is valid; it must return true to indicate that
+the program should proceed with the connection, #f to abort the connection
+attempt."
+  ;; https://libgit2.org/libgit2/#HEAD/group/callback/git_transport_certificate_check_cb
+  (let ((ptr (procedure->pointer ffi:int
+                                 (lambda (certificate valid host payload)
+                                   ;; Note: In theory the callback can also
+                                   ;; return a strictly positive number "to
+                                   ;; indicate that the callback refused to
+                                   ;; act and that the existing validity
+                                   ;; determination should be honored".  That
+                                   ;; extra value doesn't bring much compared
+                                   ;; to return VALID as is, so we ignore it.
+                                   (if (proc (pointer->string host)
+                                             (not (= 0 valid)))
+                                       0
+                                       -1))
+                                 `(* ,ffi:int * *))))
+    (hashq-set! %weak-references callbacks ptr)
+    (bytestructure-set! (remote-callbacks-bytestructure callbacks)
+                        'certificate-check (pointer-address ptr))))
+
+
 (define (fetch-options-proxy-options fetch-options)
   "Return the <proxy-options> record associated with FETCH-OPTIONS."
   (let ((bs (fetch-options-bytestructure fetch-options)))
     (%make-proxy-options (bytestructure-ref bs 'proxy-opts))))
 
 
-;; git clone options
+;; git checkout options
 
 (define %checkout-options
   (bs:struct `((version ,unsigned-int)
@@ -802,6 +870,21 @@ indexer progress record.  PROC can cancel the on-going transfer by returning
                (perfdata-cb ,(bs:pointer uint8))
                (perfdata-payload ,(bs:pointer uint8)))))
 
+(define-record-type <checkout-options>
+  (%make-checkout-options bytestructure)
+  checkout-options?
+  (bytestructure checkout-options-bytestructure))
+
+(define (make-checkout-options-bytestructure)
+  (%make-checkout-options (bytestructure %checkout-options)))
+
+(define (checkout-options->pointer checkout-options)
+  (bytestructure->pointer (checkout-options-bytestructure checkout-options)))
+
+
+
+;; git clone options
+
 (define %clone-options
   (bs:struct `((version ,int)
                (checkout-opts ,%checkout-options)
@@ -895,8 +978,10 @@ indexer progress record.  PROC can cancel the on-going transfer by returning
       (let ((bs (pointer->bytestructure pointer %remote-head)))
         (%make-remote-head
          (bytestructure-ref bs 'local)
-         (%make-oid (bytestructure-bytevector (bytestructure-ref bs 'oid)))
-         (%make-oid (bytestructure-bytevector (bytestructure-ref bs 'loid)))
+	 (pointer->oid pointer (bytestructure-offset
+				(bytestructure-ref bs 'oid)))
+	 (pointer->oid pointer (bytestructure-offset
+				(bytestructure-ref bs 'loid)))
          (pointer->string (make-pointer (bytestructure-ref bs 'name)))
          (bytestructure-ref bs 'symref-target)))))
 
@@ -1002,6 +1087,9 @@ indexer progress record.  PROC can cancel the on-going transfer by returning
                (payload ,(bs:pointer void))
                (context-lines ,uint32)
                (interhunk-lines ,uint32)
+               ,@(if %have-diff-options-oid-type?
+                     `((oid-type ,int))           ;git_oid_t
+                     '())
                (id-abbrev ,uint16)
                (max-size ,int64) ;git_off_t
                (old-prefix ,(bs:pointer uint8)) ;char*
